@@ -304,11 +304,6 @@ static int ath_reset_internal(struct ath_softc *sc, struct ath9k_channel *hchan)
 		hchan = ah->curchan;
 	}
 
-	if (!hchan) {
-		fastcc = false;
-		hchan = ath9k_cmn_get_channel(sc->hw, ah, &sc->cur_chan->chandef);
-	}
-
 	if (!ath_prepare_reset(sc))
 		fastcc = false;
 
@@ -530,10 +525,8 @@ irqreturn_t ath_isr(int irq, void *dev)
 	ath9k_debug_sync_cause(sc, sync_cause);
 	status &= ah->imask;	/* discard unasked-for bits */
 
-	if (test_bit(ATH_OP_HW_RESET, &common->op_flags)) {
-		ath9k_hw_kill_interrupts(sc->sc_ah);
+	if (test_bit(ATH_OP_HW_RESET, &common->op_flags))
 		return IRQ_HANDLED;
-	}
 
 	/*
 	 * If there are no status bits set, then this interrupt was not
@@ -816,7 +809,7 @@ static void ath9k_tx(struct ieee80211_hw *hw,
 
 	if (ath_tx_start(hw, skb, &txctl) != 0) {
 		ath_dbg(common, XMIT, "TX failed\n");
-		TX_STAT_INC(txctl.txq->axq_qnum, txfailed);
+		TX_STAT_INC(sc, txctl.txq->axq_qnum, txfailed);
 		goto exit;
 	}
 
@@ -825,80 +818,12 @@ exit:
 	ieee80211_free_txskb(hw, skb);
 }
 
-static bool ath9k_txq_list_has_key(struct list_head *txq_list, u32 keyix)
-{
-	struct ath_buf *bf;
-	struct ieee80211_tx_info *txinfo;
-	struct ath_frame_info *fi;
-
-	list_for_each_entry(bf, txq_list, list) {
-		if (bf->bf_state.stale || !bf->bf_mpdu)
-			continue;
-
-		txinfo = IEEE80211_SKB_CB(bf->bf_mpdu);
-		fi = (struct ath_frame_info *)&txinfo->rate_driver_data[0];
-		if (fi->keyix == keyix)
-			return true;
-	}
-
-	return false;
-}
-
-static bool ath9k_txq_has_key(struct ath_softc *sc, u32 keyix)
-{
-	struct ath_hw *ah = sc->sc_ah;
-	int i;
-	struct ath_txq *txq;
-	bool key_in_use = false;
-
-	for (i = 0; !key_in_use && i < ATH9K_NUM_TX_QUEUES; i++) {
-		if (!ATH_TXQ_SETUP(sc, i))
-			continue;
-		txq = &sc->tx.txq[i];
-		if (!txq->axq_depth)
-			continue;
-		if (!ath9k_hw_numtxpending(ah, txq->axq_qnum))
-			continue;
-
-		ath_txq_lock(sc, txq);
-		key_in_use = ath9k_txq_list_has_key(&txq->axq_q, keyix);
-		if (sc->sc_ah->caps.hw_caps & ATH9K_HW_CAP_EDMA) {
-			int idx = txq->txq_tailidx;
-
-			while (!key_in_use &&
-			       !list_empty(&txq->txq_fifo[idx])) {
-				key_in_use = ath9k_txq_list_has_key(
-					&txq->txq_fifo[idx], keyix);
-				INCR(idx, ATH_TXFIFO_DEPTH);
-			}
-		}
-		ath_txq_unlock(sc, txq);
-	}
-
-	return key_in_use;
-}
-
-static void ath9k_pending_key_del(struct ath_softc *sc, u8 keyix)
-{
-	struct ath_hw *ah = sc->sc_ah;
-	struct ath_common *common = ath9k_hw_common(ah);
-
-	if (!test_bit(keyix, ah->pending_del_keymap) ||
-	    ath9k_txq_has_key(sc, keyix))
-		return;
-
-	/* No more TXQ frames point to this key cache entry, so delete it. */
-	clear_bit(keyix, ah->pending_del_keymap);
-	ath_key_delete(common, keyix);
-}
-
 static void ath9k_stop(struct ieee80211_hw *hw)
 {
 	struct ath_softc *sc = hw->priv;
 	struct ath_hw *ah = sc->sc_ah;
 	struct ath_common *common = ath9k_hw_common(ah);
 	bool prev_idle;
-	int i;
 
 	ath9k_deinit_channel_context(sc);
 
@@ -966,14 +891,6 @@ static void ath9k_stop(struct ieee80211_hw *hw)
 
 	spin_unlock_bh(&sc->sc_pcu_lock);
 
-	for (i = 0; i < ATH_KEYMAX; i++)
-		ath9k_pending_key_del(sc, i);
-
-	/* Clear key cache entries explicitly to get rid of any potentially
-	 * remaining keys.
-	 */
-	ath9k_cmn_init_crypto(sc->sc_ah);
-
 	ath9k_ps_restore(sc);
 
 	sc->ps_idle = prev_idle;
@@ -1540,9 +1457,6 @@ static int ath9k_config(struct ieee80211_hw *hw, u32 changed)
 		ath_chanctx_set_channel(sc, ctx, &hw->conf.chandef);
 	}
 
-	if (changed & IEEE80211_CONF_CHANGE_POWER)
-		ath9k_set_txpower(sc, NULL);
-
 	mutex_unlock(&sc->mutex);
 	ath9k_ps_restore(sc);
 
@@ -1619,11 +1533,12 @@ static void ath9k_del_ps_key(struct ath_softc *sc,
 {
 	struct ath_common *common = ath9k_hw_common(sc->sc_ah);
 	struct ath_node *an = (struct ath_node *) sta->drv_priv;
+	struct ieee80211_key_conf ps_key = { .hw_key_idx = an->ps_key };
 
 	if (!an->ps_key)
 	    return;
 
-	ath_key_delete(common, an->ps_key);
+	ath_key_delete(common, &ps_key);
 	an->ps_key = 0;
 	an->key_idx[0] = 0;
 }
@@ -1785,12 +1700,6 @@ static int ath9k_set_key(struct ieee80211_hw *hw,
 	if (sta)
 		an = (struct ath_node *)sta->drv_priv;
 
-	/* Delete pending key cache entries if no more frames are pointing to
-	 * them in TXQs.
-	 */
-	for (i = 0; i < ATH_KEYMAX; i++)
-		ath9k_pending_key_del(sc, i);
-
 	switch (cmd) {
 	case SET_KEY:
 		if (sta)
@@ -1820,15 +1729,7 @@ static int ath9k_set_key(struct ieee80211_hw *hw,
 		}
 		break;
 	case DISABLE_KEY:
-		if (ath9k_txq_has_key(sc, key->hw_key_idx)) {
-			/* Delay key cache entry deletion until there are no
-			 * remaining TXQ frames pointing to this entry.
-			 */
-			set_bit(key->hw_key_idx, sc->sc_ah->pending_del_keymap);
-			ath_hw_keysetmac(common, key->hw_key_idx, NULL);
-		} else {
-			ath_key_delete(common, key->hw_key_idx);
-		}
+		ath_key_delete(common, key);
 		if (an) {
 			for (i = 0; i < ARRAY_SIZE(an->key_idx); i++) {
 				if (an->key_idx[i] != key->hw_key_idx)
@@ -2491,7 +2392,8 @@ out:
 	return ret;
 }
 
-static int ath9k_cancel_remain_on_channel(struct ieee80211_hw *hw)
+static int ath9k_cancel_remain_on_channel(struct ieee80211_hw *hw,
+					  struct ieee80211_vif *vif)
 {
 	struct ath_softc *sc = hw->priv;
 	struct ath_common *common = ath9k_hw_common(sc->sc_ah);
